//-----------------------------------------------------------------
// Objeto cSprite
// Archivo de encabezado - cSprite.h
//-----------------------------------------------------------------

#pragma once

//-----------------------------------------------------------------
// Archivos de inclusin
//-----------------------------------------------------------------
#include <windows.h>
#include "cBMP.h"
#include <math.h>

//-----------------------------------------------------------------
// Constantes para control de hit
//-----------------------------------------------------------------
typedef WORD        HITLEVEL;
const   HITLEVEL    HT_NO   = 0,
                    HT_NOHOTSPOT   = 1,
                    HT_HOTSPOT = 2;
//-----------------------------------------------------------------
// Constantes de opciones al llegar al final del limite (AFL_)
//-----------------------------------------------------------------
const   WORD        AFL_DETENER   = 0,
                    AFL_REBOTAR   = 1,
                    AFL_REAPARECER = 2,
                    AFL_DESAPARECER = 3,
                    AFL_REINICIAR = 4;

//-----------------------------------------------------------------
// Clase cSprite
//-----------------------------------------------------------------
class cSprite
{
protected:
  // Variables miembro protegidos

  cBMP*         m_pcBmp;

  // Rectngulo externo del mapa de bits
  RECT          m_rcSprite;

  // Rectngulo interior del mapa de bits
  // se utiliza para deteccin de colisin
  // Los puntos son relativos al
  // rectngulo externo
  RECT          m_rcHotSpotDef;

  // Posicin inicial del sprite
  POINT         m_ptInicial;

  // Velocidad en eje x y velocidad en eje y
  // expresado con un objeto POINT
  POINT         m_ptVelocidad;

  // Valor entero representa orden z
  // valor 0 es el fondo y un valor mayor se superpone
  int           m_iOrdenZ;

  // rea rectangular por donde se permite
  // el movimiento del sprite
  RECT          m_rcAreaAccion;

  // Definicin de accin automtica del sprite al
  // tocar lmites
  WORD          m_wAccionFinArea;

public:
  // Constructores y Destructor

  // Este constructor asume
  // area de accin = pantalla del juego
  // posicin inicial = 0, 0
  // accin al llegar al final del rea = se detiene
  cSprite(cBMP* pCBmp);

  cSprite(cBMP* pCBmp,
    RECT& rcAreaAccion,
    WORD wAccionFinArea = AFL_DETENER);

  // Constructor para indicar una posicin inicial
  // informada como un rectngulo y los
  // principales parmetros del objeto cSprite
  cSprite(cBMP* pCBmp,
    RECT& rcSprite,
    RECT& rcHotSpot,
    POINT  ptVelocidad,
    int iOrdenZ,
    RECT& rcAreaAccion,
    WORD wAccionFinArea = AFL_DETENER);

  cSprite(cBMP* pCBmp,
    POINT ptPosition,
    RECT& rcHotSpot,
    POINT ptVelocidad,
    int iOrdenZ,
    RECT& rcAreaAccion,
    WORD wAccionFinArea = AFL_DETENER);

  virtual ~cSprite();

  // Mtodos
  virtual void  ActualizarPosicion();
  void          Dibujar(HDC hDC);
  HITLEVEL      HitTest(int x, int y);

  // Mtodos de acceso a las propiedades
  POINT   ObtenerPosicionInicial()      { return m_ptInicial; };

  // Posicin
  RECT&   ObtenerPosicion()             { return m_rcSprite; };

  // Tres modos de asignar posicin: RECT, POINT y coordenadas x y
  void    AsignarPosicion(POINT ptPosicion);
  void    AsignarPosicion(int x, int y);
  void    AsignarPosicion(RECT& rcPosition)
    { CopyRect(&m_rcSprite, &rcPosition); };


  // Velocidad
  POINT   ObtenerVelocidad()             { return m_ptVelocidad; };

  // Dos modos de asignar velocidad: x-y  o POINT
  void    AsignarVelocidad(int x, int y);
  void    AsignarVelocidad(POINT ptVelocidad);

  // Orden Z
  int     ObtenerOrdenZ()               { return m_iOrdenZ; };
  void    AsignarOrdenZ(int iOrdenZ)    { m_iOrdenZ = iOrdenZ; };

  // Control de lmites
  void    AsignarAreaAccion(RECT& rcAreaAccion) { CopyRect(&m_rcAreaAccion, &rcAreaAccion); };
  void    AsignarAccionFinArea(WORD afl) { m_wAccionFinArea = afl; };

  // Tamao del sprite
  // son los valores del mapa de bits cBmp
  int     ObtenerAncho()                { return m_pcBmp->ObtenerAncho(); };
  int     ObtenerAltura()               { return m_pcBmp->ObtenerAltura(); };
};

