//-----------------------------------------------------------------
// Aplicacin de prueba de la clase cBMP
// Archivo de cdigo C++ - Colisiones.cpp
//-----------------------------------------------------------------

//-----------------------------------------------------------------
// Archivos de inclusin
//-----------------------------------------------------------------
#include "Colisiones.h"
#include <math.h>

//-----------------------------------------------------------------
// Funciones del NucleoMotor
//-----------------------------------------------------------------
BOOL FJ_InicializarJuego(HINSTANCE hInstancia)
{
  // Crea el ncleo del juego
  g_pJuego = new NucleoMotor(hInstancia,
    TEXT("Colisiones"), TEXT("Colisiones"),
    IDI_COLISIONES, IDI_COLISIONES_SM,
    640, 480, 15); // frecuencia fps

  if (g_pJuego == NULL)
    return false;

  // Se inicializa el joystick
  if (!g_pJuego->InicializarJoystick())
    return false;

  return true;
}

void FJ_ComenzarJuego(HWND hWindow)
{
  // Crea una serie de mapa de bits
  HDC hDC = GetDC(hWindow);

  // Crea un mapa de bits a partir de un recurso
  // Los mapas de bits son de profundidad 24 bits
  g_pPelota[0] = new cBMP(hDC, TEXT("Recursos//1.bmp"));
  g_pPelota[1] = new cBMP(hDC, TEXT("Recursos//2.bmp"));
  g_pPelota[2] = new cBMP(hDC, TEXT("Recursos//3.bmp"));
  g_pPelota[3] = new cBMP(hDC, TEXT("Recursos//4.bmp"));
  g_pPelota[4] = new cBMP(hDC, TEXT("Recursos//5.bmp"));
  g_pPelota[5] = new cBMP(hDC, TEXT("Recursos//6.bmp"));
  g_pPelota[6] = new cBMP(hDC, TEXT("Recursos//7.bmp"));
  g_pPelota[7] = new cBMP(hDC, TEXT("Recursos//8.bmp"));
  g_pPelota[8] = new cBMP(hDC, TEXT("Recursos//9.bmp"));

  g_pFondoJuego = new cBMP(hDC, IDB_BMP1, g_pJuego->ObtenerInstancia());

  // Se define un rectngulo para el lmite
  // del movimiento de los sprites: toda la pantalla
  RECT rcAreaAccion = {0, 0, g_pJuego->ObtenerAncho(), g_pJuego->ObtenerAltura()};

  g_iQSprites = 9;
  for (int i = 0; i < g_iQSprites;i++)
  {
    g_pSprite[i] = new cSprite(g_pPelota[i], rcAreaAccion, AFL_REBOTAR);
    g_pSprite[i]->AsignarVelocidad(i*3+1, i*4+1);
    g_iMaxOrdenZ = max(g_pSprite[i]->ObtenerOrdenZ(),g_iMaxOrdenZ);
  }


  // Contamos la cantidad de ciclos
  g_iContBucle = 0;


  // Bfer auxiliar de repintado
  // Crea un contexto de dispositivo y un
  // mapa de bits adicional
  g_hBuferAuxDC = CreateCompatibleDC(hDC);
  g_hBuferAuxBitmap = CreateCompatibleBitmap(hDC,
      g_pJuego->ObtenerAncho(), g_pJuego->ObtenerAltura());
  SelectObject(g_hBuferAuxDC , g_hBuferAuxBitmap);
}

void FJ_FinalizarJuego()
{
  // Elimina los elementos del bfer auxiliar
  DeleteObject(g_hBuferAuxBitmap);
  DeleteObject(g_hBuferAuxDC);

  // Elimina objetos matriz
  for (int i = 0; i < g_iQSprites ;i++)
  {
    delete g_pSprite[i];
    delete g_pPelota[i];
  }
  // elimina el mapa de bits de fondo
  delete g_pFondoJuego;

  // Elimina el objeto
  delete g_pJuego;
}

void FJ_ReactivarJuego(HWND hWindow)
{
    // Capturar el joystick
     g_pJuego->CapturarJoystick();
}

void FJ_DetenerJuego(HWND hWindow)
{
    // Liberar el joystick
     g_pJuego->LiberarJoystick();
}

void FJ_PintarJuego(HDC hDC)
{

    POINT ptOrigen = {0, 0};
    // El fondo del juego se pinta primero
    g_pFondoJuego->Dibujar(hDC, ptOrigen);

    // Despus se pintan los sprites en orden Z
    // de 0 en adelante
    for (int z = 0; z < (g_iMaxOrdenZ+1); z++)
        for (int i = 0; i < g_iQSprites; i++)
           if (g_pSprite[i]->ObtenerOrdenZ() == z)
               g_pSprite[i]->Dibujar(hDC);
}

void FJ_CicloJuego()
{
    // Actualiza las posiciones de los sprites
     for (int i = 0; i < g_iQSprites; i++)
        g_pSprite[i]->ActualizarPosicion();

    ++g_iContBucle;
     FJ_PintarJuego(g_hBuferAuxDC);

    HWND hVentana = g_pJuego->ObtenerVentana();
    HDC hDC = GetDC(hVentana);

    // se utilizan coordenadas 0, 0
    // porque se realiza copia completa
    // de un rea a otra
    BitBlt(hDC,
        0, 0, g_pJuego->ObtenerAncho(), g_pJuego->ObtenerAltura(),
        g_hBuferAuxDC, 0, 0,
        SRCCOPY);

    ReleaseDC(hVentana, hDC);
}
void FJ_LButtonDown(LPARAM lParam)
{
    POINT ptCoord = {LOWORD(lParam), HIWORD(lParam)};

  // Actualiza las posiciones de los sprites
     for (int i = 0; i < g_iQSprites; i++)
     {
       HITLEVEL hlHit = g_pSprite[i]->HitTest(ptCoord.x, ptCoord.y);
       switch (hlHit)
       {
           case HT_NOHOTSPOT:
                g_pSprite[i]->AsignarPosicion(0,0);
                break;
           case HT_HOTSPOT:
                g_pSprite[i]->AsignarPosicion(600,0);
                break;
        }
     }
}
void FJ_RButtonUp(LPARAM lParam)
{
}
void FJ_LButtonUp(LPARAM lParam)
{
}
void FJ_MouseMove(LPARAM lParam)
{
}

void FJ_RButtonDown(LPARAM lParam)
{
}

void FJ_CheckTeclado()
{
}
void FJ_CheckJoystick(WORD wJoyEstado)
{
}

