//-----------------------------------------------------------------
// Objeto cBMP
// Archivo de cdigo C++  - cBMP.cpp
//-----------------------------------------------------------------

//-----------------------------------------------------------------
// Archivos de inclusin
//-----------------------------------------------------------------
#include "cBMP.h"

//-----------------------------------------------------------------
// Constructor(s)/Destructor
//-----------------------------------------------------------------
cBMP::cBMP() : m_hcBMP(NULL)
{
}

// Crea un mapa de bits vaco
cBMP::cBMP(HDC hDC, SIZE sizBmp, COLORREF crColor) : m_hcBMP(NULL)
{
  CrearBMP(hDC, sizBmp, crColor);
}

// Crea un mapa de bits a partir de un archivo bmp
cBMP::cBMP(HDC hDC, LPTSTR szFileName) : m_hcBMP(NULL)
{
  CrearBMP(hDC, szFileName);
}

// Crea un mapa de bits a partir de un recurso bmp
cBMP::cBMP(HDC hDC, UINT uiIDRecurso, HINSTANCE hInstancia) : m_hcBMP(NULL)
{
  CrearBMP(hDC, uiIDRecurso, hInstancia);
}

// Destructor
cBMP::~cBMP()
{
  Liberar();
}

//-----------------------------------------------------------------
// Mtodos auxiliares
//-----------------------------------------------------------------
void cBMP::Liberar()
{
  // Elimina el objeto
  if (m_hcBMP != NULL)
  {
    DeleteObject(m_hcBMP);
    m_hcBMP = NULL;
  }
}

//-----------------------------------------------------------------
// Mtodos generales
//-----------------------------------------------------------------
BOOL cBMP::CrearBMP(HDC hDC, LPTSTR szFileName)
{
  // liberar por si acaso
  Liberar();

  // Abre el archivo
  HANDLE hFile = CreateFile(szFileName,
    GENERIC_READ, FILE_SHARE_READ, NULL,
    OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

  // si no se obtiene el handle
  // se termina la funcin
  if (hFile == INVALID_HANDLE_VALUE)
    return false;

  // Lectura del encabezado del archivo mapa de bits
  BITMAPFILEHEADER  bmfHeader;
  DWORD             dwBytesRead;

  BOOL bOK = ReadFile(hFile,
    &bmfHeader, sizeof(BITMAPFILEHEADER),
    &dwBytesRead, NULL);

  if ((!bOK) || (dwBytesRead != sizeof(BITMAPFILEHEADER)) ||
    (bmfHeader.bfType != 0x4D42))
  {
    //0x4D42 indica que es un bitmap
    // siempre debe tener ese valor
    // si no-> fin de la funcin
    CloseHandle(hFile);
    return false;
  }

// Lectura de la informacin del encabezado
  BITMAPINFO* pcBMPInfo = (new BITMAPINFO);
  if (pcBMPInfo != NULL)
  {
    // Lee la informacin del encabezado
    bOK = ReadFile(hFile, pcBMPInfo, sizeof(BITMAPINFOHEADER),
          &dwBytesRead, NULL);
    if ((!bOK) || (dwBytesRead != sizeof(BITMAPINFOHEADER)))
    {
      // Si la lectura no es correcta
      // finaliza la funcin
      CloseHandle(hFile);
      return false;
    }

    // Almacena el tamao del mapa de bits
    // en las variables miembro del objeto
    m_eInfoHeader =  pcBMPInfo->bmiHeader;

    // Puntero a byte
    BYTE * pcBMPBits;

    // DIB_RGB_COLORS indica que BITMAPINFO
    // contiene una matriz RGB()
    // se informa la direccin en donde estar
    // los bytes del bitmap, pero an
    // no se han ledo
    m_hcBMP = CreateDIBSection(hDC,
        pcBMPInfo, DIB_RGB_COLORS,
        (PVOID*)&pcBMPBits, NULL, 0);


    // Copia los bits
    if ((m_hcBMP != NULL) && (pcBMPBits != NULL))
    {
      // Coloca el puntero al inicio del archivo
      SetFilePointer(hFile, bmfHeader.bfOffBits, NULL, FILE_BEGIN);
      // Aqu se lee el archivo y se copian los bytes
      // en PcBMPBits
      bOK = ReadFile(hFile, pcBMPBits, pcBMPInfo->bmiHeader.biSizeImage,
                    &dwBytesRead, NULL);
      if (bOK)
        return true;
    }
  }

  // Libera los recursos
  Liberar();
  return false;
}

BOOL cBMP::CrearBMP(HDC hDC, UINT uiIDRecurso, HINSTANCE hInstancia)
{
  // Libera recursos anteriores
  Liberar();

  // Busca el recurso
  HRSRC hResInfo = FindResource(hInstancia,
    MAKEINTRESOURCE(uiIDRecurso), RT_BITMAP);

// Si no obitiene el handle, termina la funcin
  if (hResInfo == NULL)
    return false;

  // Carga el recurso en memoria global
  HGLOBAL hMemcBMP = LoadResource(hInstancia, hResInfo);
  if (hMemcBMP == NULL)
    return false;

  // Bloquea el recurso
  // la funcin LockResource devuelve un puntero al
  // primer byte del recurso (NULL si no pudo bloquear)
  BYTE * pcBMPImage = (BYTE*)LockResource(hMemcBMP);
  if (pcBMPImage == NULL)
    return false;


  // Almacena la informacin del mapa de bits
  BITMAPINFO* pcBMPInfo = (BITMAPINFO*)pcBMPImage;
  m_eInfoHeader =  pcBMPInfo->bmiHeader;

  // Obtiene el handle y copia los bits
  BYTE * pcBMPBits;

  // DIB_RGB_COLORS indica que BITMAPINFO
  // contiene una matriz RGB()
  // se informa la direccin en donde estar
  // los bytes del bitmap, pero an
  // no se han ledo
  m_hcBMP = CreateDIBSection(hDC,
        pcBMPInfo, DIB_RGB_COLORS,
        (PVOID*)&pcBMPBits, NULL, 0);

  if ((m_hcBMP != NULL) && (pcBMPBits != NULL))
  {
    // se calcula la direccin en donde
    // empiezan los datos del mapa de bits
    const  BYTE * pBitsOrigen = pcBMPImage +    // puntero a primera posicin
        pcBMPInfo->bmiHeader.biSize +       // tamao del header
                                            // cantidad de colores X tamao RGBQUAD
        pcBMPInfo->bmiHeader.biClrUsed * sizeof(RGBQUAD);

    // Copia el bloque de memoria
    CopyMemory(pcBMPBits, pBitsOrigen, pcBMPInfo->bmiHeader.biSizeImage);

     return true;
  }

  // Limpieza
  Liberar();
  return false;
}

BOOL cBMP::CrearBMP(HDC hDC, SIZE sizBmp, COLORREF crColor)
{
  // Crea un mapa de bits vaco
  m_hcBMP = CreateCompatibleBitmap(hDC, sizBmp.cx, sizBmp.cy);
  if (m_hcBMP == NULL)
    return false;

  // Almacena el tamao en el miembro del objeto
  m_eInfoHeader.biWidth = sizBmp.cx;
  m_eInfoHeader.biHeight = sizBmp.cy;

  // Crea un contexto de dispositivo en memoria
  // para dibujar el mapa de bits
  HDC hMemDC = CreateCompatibleDC(hDC);

  // Crea una brocha uniforme (slida) para rellenar el mapa de bits
  HBRUSH hBrush = CreateSolidBrush(crColor);

  // Selecciona el mapa de bits en el contexto de memoria
  HBITMAP hcBMPAnterior = (HBITMAP)SelectObject(hMemDC, m_hcBMP);

  // Rellena el mapa de bits
  RECT rccBMP = { 0, 0, ObtenerAncho(), ObtenerAltura() };
  // Rellena el rectngulo del objeto seleccionado (el mapa de bits)
  // con la brocha definida
  FillRect(hMemDC, &rccBMP, hBrush);

  // Limpieza
  SelectObject(hMemDC, hcBMPAnterior);
  DeleteDC(hMemDC);
  DeleteObject(hBrush);

  return true;
}

void cBMP::Dibujar(HDC hDC, POINT ptInicio, BOOL bTransp, COLORREF crTranspColor)
{
  POINT ptOrigen = {0,0};
  SIZE sizTotal;
  sizTotal.cx = ObtenerAncho();
  sizTotal.cy = ObtenerAltura();
  // Los dibujos se hacen todos en la funcin DibujoParcial
  DibujoParcial(hDC, ptInicio, ptOrigen , sizTotal, bTransp, crTranspColor);
}

void cBMP::Dibujar(HDC hDC, int iX, int iY, BOOL bTransp, COLORREF crTranspColor)
{
  POINT ptOrigen = {0,0};
  POINT ptInicio = {iX, iY};
  SIZE sizTotal;
  sizTotal.cx = ObtenerAncho();
  sizTotal.cy = ObtenerAltura();
  // Los dibujos se hacen todos en la funcin DibujoParcial
  DibujoParcial(hDC, ptInicio, ptOrigen , sizTotal, bTransp, crTranspColor);
}

void cBMP::DibujoParcial(HDC hDC, POINT ptInicio, POINT ptOrigen,
                         SIZE sizParcial, BOOL bTransp, COLORREF crTranspColor)
{
  if (m_hcBMP != NULL)
  {
    // Crea un contexto de dispositivo compatible
    HDC hDCMem = CreateCompatibleDC(hDC);

    // Selecciona el mapa de bits en el contexto
    HBITMAP hcBMPAnterior = (HBITMAP)SelectObject(hDCMem, m_hcBMP);

    // Dibuja el mapa de bits en el contexto del dispositivo

    if (bTransp)
       TransparentBlt(hDC, ptInicio.x, ptInicio.y, sizParcial.cx, sizParcial.cy,
                      hDCMem, ptOrigen.x, ptOrigen.y, sizParcial.cx, sizParcial.cy,
                      crTranspColor);
    else
       BitBlt(hDC, ptInicio.x, ptInicio.y,
              sizParcial.cx, sizParcial.cy, hDCMem,
              ptOrigen.x, ptOrigen.y, SRCCOPY);

    // Restaura el objeto anterior
    SelectObject(hDCMem, hcBMPAnterior);
    // Libera el contexto del dispositivo
    DeleteDC(hDCMem);
  }
}

