//-----------------------------------------------------------------
// Archivo de cdigo C++ - UsoJoystick.cpp
//-----------------------------------------------------------------

//-----------------------------------------------------------------
// Archivos de inclusin
//-----------------------------------------------------------------
#include "UsoJoystick.h"
#include <math.h>

//-----------------------------------------------------------------
// Funciones del NucleoMotor
//-----------------------------------------------------------------
BOOL FJ_InicializarJuego(HINSTANCE hInstancia)
{
  // Crea el ncleo del juego
  g_pJuego = new NucleoMotor(hInstancia,
    TEXT("UsoJoystick"), TEXT("UsoJoystick"),
    IDI_TESTCBMP, IDI_TESTCBMP_SM,
    600, 450, 10); // frecuencia 50 fps

  if (g_pJuego == NULL)
    return false;

  // Se inicializa el joystick
  if (!g_pJuego->InicializarJoystick())
    return false;

  return true;
}

void FJ_ComenzarJuego(HWND hWindow)
{
  // Crea una serie de mapa de bits
  HDC hDC = GetDC(hWindow);


  // Crea un mapa de bits a partir de un recurso
  // Los mapas de bits son de profundidad 24 bits
  g_pOvni = new cBMP(hDC, TEXT("Recursos//Ovni.bmp"));
  g_pFondoJuego = new cBMP(hDC, IDB_BMP1, g_pJuego->ObtenerInstancia());


 // Posicin inicial del objeto
  g_ptOvni.x = (g_pJuego->ObtenerAncho()) / 2 - (g_pOvni->ObtenerAncho() / 2);
  g_ptOvni.y = 0;

  // La direccin inicial del ovni es hacia abajo
  g_ptDireccion.x =  g_ptOvni.x;
  g_ptDireccion.y =  g_pJuego->ObtenerAltura();

  // Velocidad del Ovni
  g_iVelocidad = g_iVelocidadMax;

  // Contamos la cantidad de ciclos
  g_iContBucle = 0;


  // Bfer auxiliar de repintado
  // Crea un contexto de dispositivo y un
  // mapa de bits adicional
  g_hBuferAuxDC = CreateCompatibleDC(hDC);
  g_hBuferAuxBitmap = CreateCompatibleBitmap(hDC,
      g_pJuego->ObtenerAncho(), g_pJuego->ObtenerAltura());
  SelectObject(g_hBuferAuxDC , g_hBuferAuxBitmap);

}

void FJ_FinalizarJuego()
{

  // Elimina los elementos del bfer auxiliar
  DeleteObject(g_hBuferAuxBitmap);
  DeleteObject(g_hBuferAuxDC);

  // Elimina objetos
  delete g_pOvni;
  delete g_pFondoJuego;

  // Elimina el objeto
  delete g_pJuego;
}

void FJ_ReactivarJuego(HWND hWindow)
{
    // Capturar el joystick
     g_pJuego->CapturarJoystick();
}

void FJ_DetenerJuego(HWND hWindow)
{
    // Liberar el joystick
     g_pJuego->LiberarJoystick();
}

void FJ_PintarJuego(HDC hDC)
{

    POINT ptOrigen = {0, 0};
    g_pFondoJuego->Dibujar(hDC, ptOrigen);;
    g_pOvni->Dibujar(hDC, g_ptOvni, true, RGB(255, 0, 255));

}

void FJ_CicloJuego()
{

    // Actualiza la posicin del OVNI
    // lgebra del problema
    // actualiza la posicin x y del ovni en base
    // a la velocidad y la direccin del objeto
    //
    // Hipotenusa = h
    int x1 = g_ptOvni.x;
    int y1 = g_ptOvni.y;
    int x2 = g_ptDireccion.x;
    int y2 = g_ptDireccion.y;

    double h = sqrt((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1));
    double b = x2 -x1;
    double a = y2 -y1;
    g_ptOvni.x += (g_iVelocidad * b /h);
    g_ptOvni.y += (g_iVelocidad * a /h);

    // ajuste de lmites para que el objeto no salga de la pantalla
    g_ptOvni.x = min(g_pJuego->ObtenerAncho() - g_pOvni->ObtenerAncho(), g_ptOvni.x);
    g_ptOvni.x = max(0, g_ptOvni.x);
    g_ptOvni.y = max(0, g_ptOvni.y);
    g_ptOvni.y = min(g_pJuego->ObtenerAltura() - g_pOvni->ObtenerAltura(), g_ptOvni.y);

    ++g_iContBucle;
    // Invalida la ventana para forzar el repintado
    ///InvalidateRect(g_pJuego->ObtenerVentana(), NULL, false);
    FJ_PintarJuego(g_hBuferAuxDC);

    HWND hVentana = g_pJuego->ObtenerVentana();
    HDC hDC = GetDC(hVentana);

    // se utilizan coordenadas 0, 0
    // porque se realiza copia completa
    // de un rea a otra
    BitBlt(hDC,
        0, 0, g_pJuego->ObtenerAncho(), g_pJuego->ObtenerAltura(),
        g_hBuferAuxDC, 0, 0,
        SRCCOPY);

    ReleaseDC(hVentana, hDC);


}
void FJ_LButtonDown(LPARAM lParam)
{

    // el botn izquierdo del ratn
    // sirve para orientar el ovni
    // esta funcin se aade
    // al uso del teclado y del joystick

    POINT ptCoord = {LOWORD(lParam), HIWORD(lParam)};
    g_ptDireccion.x = ptCoord.x;
    g_ptDireccion.y = ptCoord.y;

}
void FJ_RButtonUp(LPARAM lParam)
{

}
void FJ_LButtonUp(LPARAM lParam)
{

}
void FJ_MouseMove(LPARAM lParam)
{

}
void FJ_RButtonDown(LPARAM lParam)
{

   // El botn derecho del ratn
   // sirve para acelerar o frenar el ovni
    if (g_iVelocidad > 0)
        g_iVelocidad = 0;
    else
        g_iVelocidad = g_iVelocidadMax;

}

void FJ_CheckTeclado()
{
    // Se comprueba si se ha aumentado o disminuido
    // la velocidad

    // Tecla SHIFT aumenta la velocidad
    if (GetAsyncKeyState(VK_SHIFT) < 0)
        g_iVelocidad += 1;
    // Tecla CONTROL disminuye la velocidad
    if (GetAsyncKeyState(VK_CONTROL) < 0)
        g_iVelocidad -= 1;
    // Se verifica que el rango de velocidad quede
    // entre 0 y mximo
    g_iVelocidad = min(g_iVelocidad, g_iVelocidadMax);
    g_iVelocidad = max(0, g_iVelocidad);

    // Las teclas de flechas sirven para indicar direccin
    // como apoyo al uso del ratn

    // Con el teclado se pueden definir 4 direcciones
    // en cambio con el ratn el ajuste es ms fino;
    // con el joystick se tienen 8 direcciones

    // Hacia la izquierda
    if (GetAsyncKeyState(VK_LEFT) < 0)
    {
         g_ptDireccion.x = 0;
         g_ptDireccion.y = g_ptOvni.y;
    }
    // Hacia arriba
    if (GetAsyncKeyState(VK_UP) < 0)
    {
         g_ptDireccion.x = g_ptOvni.x;
         g_ptDireccion.y = 0;
    }
    // Hacia la derecha
    if (GetAsyncKeyState(VK_RIGHT) < 0)
    {
        g_ptDireccion.x = g_pJuego->ObtenerAncho();
        g_ptDireccion.y = g_ptOvni.y;
    }
    // Hacia abajo
    if (GetAsyncKeyState(VK_DOWN) < 0)
    {
        g_ptDireccion.x = g_ptOvni.x;
        g_ptDireccion.y = g_pJuego->ObtenerAltura();
    }


}
void FJ_CheckJoystick(WORD wJoyEstado)
{

  // El joystick permite una definicin de 8 direcciones en el
  // plano x y
  // con 3 botones (1, 2 y 3) se controla
  // la velocidad

  // Verificar el movimiento vertical
  if (wJoyEstado & JARRIBA)
  {
     g_ptDireccion.x = g_ptOvni.x;
     g_ptDireccion.y = 0;
  }
  else if (wJoyEstado & JABAJO)
  {
        g_ptDireccion.x = g_ptOvni.x;
        g_ptDireccion.y = g_pJuego->ObtenerAltura();
  }

  // Verifica el movimiento horizontal
  // y el combinado
  if (wJoyEstado & JIZQ)
  {
      g_ptDireccion.x = 0;
      g_ptDireccion.y = g_ptOvni.y;
    if (wJoyEstado & JARRIBA)
        g_ptDireccion.y = 0;
    if (wJoyEstado & JABAJO)
        g_ptDireccion.y = g_pJuego->ObtenerAltura();
   }
  if (wJoyEstado & JDER)
  {
    g_ptDireccion.x = g_pJuego->ObtenerAncho();
    g_ptDireccion.y = g_ptOvni.y;
     if (wJoyEstado & JARRIBA)
         g_ptDireccion.y = 0;
     if (wJoyEstado & JABAJO)
        g_ptDireccion.y = g_pJuego->ObtenerAltura();
    }

  // Se comprueba si se ha aumentado o disminuido
  // la velocidad
  // Botn 1 aumenta la velocidad
  if (wJoyEstado & JBOT1)
      g_iVelocidad += 1;
  // Botn 2 disminuye la velocidad
  if (wJoyEstado & JBOT2)
        g_iVelocidad -= 1;
  // Botn 3 frena el ovni
  if (wJoyEstado & JBOT3)
        g_iVelocidad = 0;
  // Se verifica que el rango de velocidad quede
  // entre 0 y mximo
  g_iVelocidad = min(g_iVelocidad, g_iVelocidadMax);
  g_iVelocidad = max(0, g_iVelocidad);

}

