//-----------------------------------------------------------------
// Objeto NucleoMotor
// Codigo fuente C++  - NucleoMotor.cpp
//-----------------------------------------------------------------

//-----------------------------------------------------------------
// Archvos de inclusion
//-----------------------------------------------------------------
#include "NucleoMotor.h"

//-----------------------------------------------------------------
// Inicializacion de variables estticas
//-----------------------------------------------------------------
NucleoMotor *NucleoMotor::m_pNucleoMotor = NULL;

//-----------------------------------------------------------------
// Funciones Windows
//-----------------------------------------------------------------
int WINAPI  WinMain(HINSTANCE hInstancia, HINSTANCE hInstanciaPrev,
                    PSTR szCmdLine, int iMostrar)
{
  MSG         msg;
  static int  iSiguienteCiclo = 0;
  int         iTicsPasados;
  srand(GetTickCount());

  if (FJ_InicializarJuego(hInstancia))
  {
    // Inicializa el ncleo motor
    // (crea la ventana y la muestra en pantalla)
    if (!NucleoMotor::ObtenerNucleoMotor()->Inicializar(iMostrar))
      return false;


    // Bucle principal de mensajes
    while (true)
    {
      if (PeekMessage(&msg, NULL, 0, 0,PM_REMOVE))
      {
        // Proceso del mensaje
        // El mensaje WM_QUIT hace salir del bucle
        // La funcin PeekMessage entrega los mensajes que
        // hay en la cola
        if (msg.message == WM_QUIT)
          break;
        // Traduce los mensajes de teclas virtuales
        TranslateMessage(&msg);

        // enva el mensaje al procedimiento de ventana
        // donde sern tratados por el mtodo TratarMensajes()
        DispatchMessage(&msg);
      }
      // cuando hay mensajes pendientes
      // verifica si ya se debe ejecutar un ciclo
      else
      {
        // Se verifica que el juego no est en pausa
        if (!NucleoMotor::ObtenerNucleoMotor()->ObtenerEnEspera())
        {
          // Se verifica la cuenta de tics para ver si
          // ha pasado el ciclo del juego
          iTicsPasados = GetTickCount();

          if (iTicsPasados > iSiguienteCiclo)
          {
            // Antes de ir al ciclo del juego
            // actualiza el tiempo del ciclo
            // siguiente
            iSiguienteCiclo = iTicsPasados + 1000 /
              NucleoMotor::ObtenerNucleoMotor()->ObtenerFrecuencia() ;

            // Verifica si hubo entrada por teclado
            FJ_CheckTeclado();

            // Verifica el estado del joystick antes de cada ciclo
            NucleoMotor::ObtenerNucleoMotor()->ComprobarJoystick();

            // Ejecucin del ciclo del juego
            FJ_CicloJuego();
          }
        }
      }
    }
    return (int)msg.wParam;
  }

  // Final del juego
  FJ_FinalizarJuego();

  return true;
}

LRESULT CALLBACK WindowProc(HWND hVentana, UINT msg, WPARAM wParam, LPARAM lParam)
{
  // Procedimiento de ventana
  // Envia los mensajes a la funcin de tratamiento de los eventos
  return NucleoMotor::ObtenerNucleoMotor()->TratarMensajes(hVentana, msg, wParam, lParam);
}

//-----------------------------------------------------------------
// NucleoMotor Constructor(s)/Destructor
//-----------------------------------------------------------------
NucleoMotor::NucleoMotor(HINSTANCE hInstancia,
        LPTSTR szClaseVentana, LPTSTR szTitulo,
        WORD wIcono, WORD wIconoSm,
        int iAncho, int iAltura,
        int iFrecuencia)
{
  // Asigna las variables miembro del objeto NucleoMotor
  m_pNucleoMotor = this;
  m_hInstancia = hInstancia;

  // Queda pendiente asignar el handle de la ventana
  m_hVentana = NULL;

  // copia la cadena cuando hay informacin
  if (lstrlen(szClaseVentana) > 0)
    lstrcpy(m_szClaseVentana, szClaseVentana);

  // copia la cadena cuando hay informacin
  if (lstrlen(szTitulo) > 0)
    lstrcpy(m_szTitulo, szTitulo);

  // Identificadores del recursos Icono
  m_wIcono = wIcono;
  m_wIconoSm = wIconoSm;

  // Dimensiones de la ventana
  m_iAncho = iAncho;
  m_iAltura = iAltura;

  // La frecuencia del ciclo es en frames por segundo
  m_iFrecuencia = iFrecuencia; // en fps

  // Identificador del joystick
  m_uiIDJoystick = 0;

  // El juego comienza en pausa
  m_bEnEspera = true;

  // Reserva espacio para una cantidad
  // inicial de 20 objetos
  m_vcSprites.reserve(20);

  m_uiMIDIPlayerID = 0;
}

NucleoMotor::~NucleoMotor()
{
    // De momento no hace nada en particular
}

//-----------------------------------------------------------------
// Mtodos del objeto NucleoMotor
//-----------------------------------------------------------------
BOOL NucleoMotor::Inicializar(int iMostrar)
{
  WNDCLASSEX    wndclass;

  // Crea la clase ventana para la ventana principal
  wndclass.cbSize         = sizeof(wndclass);
  wndclass.style          = CS_HREDRAW | CS_VREDRAW;
  wndclass.lpfnWndProc    = WindowProc;
  wndclass.cbClsExtra     = 0;
  wndclass.cbWndExtra     = 0;
  wndclass.hInstance      = m_hInstancia;
  wndclass.hIcon          = LoadIcon(m_hInstancia,
    MAKEINTRESOURCE(ObtenerIcono()));
  wndclass.hIconSm        = LoadIcon(m_hInstancia,
    MAKEINTRESOURCE(ObtenerIconoSm()));
  wndclass.hCursor        = LoadCursor(NULL, IDC_ARROW);
  wndclass.hbrBackground  = (HBRUSH)(COLOR_WINDOW+1);
  wndclass.lpszMenuName   = NULL;
  wndclass.lpszClassName  = m_szClaseVentana;

  // Registra la clase de ventanas
  if (!RegisterClassEx(&wndclass))
      return false;


  // Calcula el tamao de la ventana
  int iAncho = m_iAncho + GetSystemMetrics(SM_CXFIXEDFRAME) * 2;
  int iAltura = m_iAltura + GetSystemMetrics(SM_CYFIXEDFRAME) * 2 +
        GetSystemMetrics(SM_CYCAPTION);

  // Verifica si hay ttulo de ventana
  // en ese caso se corrige la altura
  if (wndclass.lpszMenuName != NULL)
    iAltura += GetSystemMetrics(SM_CYMENU);

  // Posicionamiento de la ventana en el centro de la pantalla
  int iPosX = (GetSystemMetrics(SM_CXSCREEN) - iAncho) / 2;
  int iPosY = (GetSystemMetrics(SM_CYSCREEN) - iAltura) / 2;

  // Crea la ventana
  // La ventana no se puede redimensionar
  // pero se puede minimizar y tiene barra de ttulo
  m_hVentana = CreateWindow(m_szClaseVentana, m_szTitulo,
    WS_POPUPWINDOW | WS_CAPTION | WS_MINIMIZEBOX,
    iPosX, iPosY,
    iAncho, iAltura,
    NULL, NULL, m_hInstancia, NULL);

  // Si no puede crear la ventana devuelve False
  if (!m_hVentana)
    return false;

  // Mostrar y actualizar ventana
  ShowWindow(m_hVentana, iMostrar);
  UpdateWindow(m_hVentana);

  return true;
}

LRESULT NucleoMotor::TratarMensajes(HWND hVentana, UINT msg, WPARAM wParam, LPARAM lParam)
{
  // Tratamiento de los mensajes
  switch (msg)
  {
  //-----------------------
    case WM_CREATE:
      // Asigna la ventana y comienza el juego
      AsignarVentana(hVentana);
      FJ_ComenzarJuego(hVentana);
      return 0;
  //-----------------------
    case WM_KILLFOCUS:
      // Desactiva el juego y actualiza el estado de pausa
      FJ_DetenerJuego(hVentana);
      AsignarEnEspera(true);
      return 0;
  //-----------------------
    case WM_PAINT:
      HDC         hDC;
      PAINTSTRUCT ps;
      hDC = BeginPaint(hVentana, &ps);

      // Dibuja el juego
      FJ_PintarJuego(hDC);

      EndPaint(hVentana, &ps);
      return 0;
  //-----------------------
    case WM_SETFOCUS:
      // Activa el juego y actualiza el estado de pausa
      FJ_ReactivarJuego(hVentana);
      AsignarEnEspera(false);
      return 0;
  //-----------------------
    case WM_DESTROY:
      // Final del juego y sale de la aplicacion
      FJ_FinalizarJuego();
      PostQuitMessage(0);
      return 0;
  //-----------------------
    case WM_MOUSEMOVE:
      // Movimiento del ratn
      FJ_MouseMove(lParam);
      return 0;
      //-----------------------
    case WM_LBUTTONDOWN:
      // Pulsa el botn izquierdo
      FJ_LButtonDown(lParam);
      return 0;
       //-----------------------
    case WM_RBUTTONDOWN:
      // Pulsa el botn derecho
      FJ_RButtonDown(lParam);
      return 0;
      //-----------------------
      //-----------------------
    case WM_LBUTTONUP:
      // Suelta el botn izquierdo
      FJ_LButtonUp(lParam);
      return 0;
      //-----------------------
    case WM_RBUTTONUP:
      // Pulsa el botn derecho
      FJ_RButtonUp(lParam);
      return 0;
      //-----------------------


  }
  // Tratamiento del resto de los mensajes
  // no gestionados por NucleoMotor
  return DefWindowProc(hVentana, msg, wParam, lParam);
}
BOOL NucleoMotor::InicializarJoystick()
{
  // Comprobacin de la cantidad de joysticks soportados
  if (joyGetNumDevs() == 0)
      return false;


  // Comprobacin de conexin
  JOYINFO jiJoyInfo;
  // en la estructura jiJoyInfo queda registrada
  // la posicin y el estado de los botones
  // JOYSTICKID1 es 0
  if (joyGetPos(JOYSTICKID1, &jiJoyInfo) != JOYERR_NOERROR)
    // joystick desconectado
    return false;

  // Guardamos el identificador del joystick
  m_uiIDJoystick = JOYSTICKID1;

  // Capacidades del joystick
  JOYCAPS jcCaps;
  joyGetDevCaps(m_uiIDJoystick, &jcCaps, sizeof(JOYCAPS));

  // Obtiene el centro
  DWORD dwXCenter = ((DWORD)jcCaps.wXmin + jcCaps.wXmax) / 2;
  DWORD dwYCenter = ((DWORD)jcCaps.wYmin + jcCaps.wYmax) / 2;

  // Define el rectngulo de tolerancia
  m_rcRectTolerancia.left = (jcCaps.wXmin + (WORD)dwXCenter) / 2;
  m_rcRectTolerancia.right = (jcCaps.wXmax + (WORD)dwXCenter) / 2;
  m_rcRectTolerancia.top = (jcCaps.wYmin + (WORD)dwYCenter) / 2;
  m_rcRectTolerancia.bottom = (jcCaps.wYmax + (WORD)dwYCenter) / 2;

  return true;
}

void NucleoMotor::CapturarJoystick()
{
  // Capturar el joystick
  // La funcin captura un joystick, lo que
  // hace que los mensajes se enven a la ventana
  if (m_uiIDJoystick == JOYSTICKID1)
    joySetCapture(m_hVentana, m_uiIDJoystick, NULL, true);
}

void NucleoMotor::LiberarJoystick()
{
  // Libera el joystick
  if (m_uiIDJoystick == JOYSTICKID1)
    joyReleaseCapture(m_uiIDJoystick);
}

void NucleoMotor::ComprobarJoystick()
{
  if (m_uiIDJoystick == JOYSTICKID1)
  {
    JOYINFO jiJoyInfo;
    WORD wJoyEstado= 0;
    // carga la estructura de informacin del joystick
    if (joyGetPos(m_uiIDJoystick, &jiJoyInfo) == JOYERR_NOERROR)
    {
      // Se comprueban 4 botones
      if(jiJoyInfo.wButtons & JOY_BUTTON1)
        wJoyEstado |= JBOT1;
      if(jiJoyInfo.wButtons & JOY_BUTTON2)
        wJoyEstado |= JBOT2;
      if(jiJoyInfo.wButtons & JOY_BUTTON3)
        wJoyEstado |= JBOT3;
      if(jiJoyInfo.wButtons & JOY_BUTTON4)
        wJoyEstado |= JBOT4;

       // Verifica el movimiento vertical
      if (jiJoyInfo.wYpos < (WORD)m_rcRectTolerancia.top)
        // la variable wJoyEstado se va modificando
        // mediante la suma lgica de bits
        wJoyEstado |= JARRIBA;
      else if (jiJoyInfo.wYpos > (WORD)m_rcRectTolerancia.bottom)
        wJoyEstado |= JABAJO;

      // Verifica el movimiento horizontal
      if (jiJoyInfo.wXpos < (WORD)m_rcRectTolerancia.left)
        wJoyEstado |= JIZQ;
      else if (jiJoyInfo.wXpos > (WORD)m_rcRectTolerancia.right)
        wJoyEstado |= JDER;
    }

    // Se pasa el control al juego
    FJ_CheckJoystick(wJoyEstado);
  }
}
//-----------------------------------------------------------------
// Mtodos para el control de colisiones
//-----------------------------------------------------------------
cSprite* NucleoMotor::DetectarColision(cSprite* pSprite)
{
  // A este mtodo se llama desde NucleoMotor::ActualizarSprites()

  // Verifica cada sprite con el resto

  // Crea un iterador para recorrer el vector
  vector<cSprite*>::iterator ivSprite;

  // Bucle por todo el vector
  for (ivSprite = m_vcSprites.begin(); ivSprite != m_vcSprites.end(); ivSprite++)
  {
    // Se omite al propio sprite
    // (en el vector estn todos los sprites)
    if (pSprite == (*ivSprite))
      continue;

    // Verifica si hay colisin
    // comparando los rectngulos de hot spot
    // del sprite vigente con el que le toque
    // comparar
    if (pSprite->VerificarColision(*ivSprite))
      // Si hay colisin
      // devuelve el puntero al objeto
       return *ivSprite;

  }

  // No hubo colisin
  return NULL;
}
void NucleoMotor::AgregarSprite(cSprite* pSprite)
{
  // Comprueba si se debe incrementar
  // la capacidad del vector
  if (m_vcSprites.size() >= (m_vcSprites.capacity() / 2))
    m_vcSprites.reserve(m_vcSprites.capacity() * 2);

  // Incluye el puntero del sprite en el vector
  if (pSprite != NULL)
  {
    // Ya hay elementos en el vector
    if (m_vcSprites.size() > 0)
    {
      // Coloca el sprite manteniendo el orden
      // del orden Z
      // Crea un iterador
      vector<cSprite*>::iterator ivSprite;

      for (ivSprite = m_vcSprites.begin(); ivSprite != m_vcSprites.end(); ivSprite++)
        if (pSprite->ObtenerOrdenZ() < (*ivSprite)->ObtenerOrdenZ())
        {
          // Inserta el sprite en el lugar correcto
          // segn el orden Z (de menor a mayor)
          m_vcSprites.insert(ivSprite, pSprite);
          return;
        }
    }

    // No haba elementos en el vector
    m_vcSprites.push_back(pSprite);
  }
}

void NucleoMotor::DibujarSprites(HDC hDC)
{
  // Dibuja todos los sprites
  // existentes en el vector

  // Crea un iterador para navegar por el vector
  vector<cSprite*>::iterator ivSprite;

  for (ivSprite = m_vcSprites.begin(); ivSprite != m_vcSprites.end(); ivSprite++)
    // Llama a la funcin Dibujar de cada sprite
    (*ivSprite)->Dibujar(hDC);
}

void NucleoMotor::ActualizarSprites()
{

  // Actualiza las posiciones
  RECT          rcPosAnterior;
  APC_ACCION    apcAccion;

  // Crea un iterador para navegar por el vector
  vector<cSprite*>::iterator ivSprite;

  // Navega por todo el vector para actualizar
  // las psoiciones de cada uno de los objetos cSprite
  for (ivSprite = m_vcSprites.begin(); ivSprite != m_vcSprites.end(); ivSprite++)
  {
    // Se mantiene la posicin vigente
    // por si hay que utilizarla
    // para restaurarla ante una colisin
    rcPosAnterior = (*ivSprite)->ObtenerPosicion();

    // Actualiza la posicin
    apcAccion = (*ivSprite)->ActualizarPosicion();

    // Destruccin del objeto
    switch (apcAccion)
    {
        case APC_DESTRUIR:
            delete (*ivSprite);
            // elimina el objeto del vector
            ivSprite = m_vcSprites.erase(ivSprite);
            //ivSprite--;
            continue;
            break;
        case APC_NOTIFICAR:
            // Notifica que un sprite ha llegado a su lmite
            FJ_NotificarColision(TC_LIMITES, *ivSprite, NULL);
            break;
    }


    // Control de colisiones con otros sprites
    cSprite* pSprite = DetectarColision(*ivSprite);
    if (pSprite != NULL)
    {
        (*ivSprite)->AsignarPosicion(rcPosAnterior);
        // Notifica al juego que hubo una colisin entre sprites
        if  (FJ_NotificarColision(TC_SPRITE, (*ivSprite), pSprite) == APC_DESTRUIR)
        {
            delete (*ivSprite);
            // elimina el objeto del vector
            ivSprite = m_vcSprites.erase(ivSprite);
            //ivSprite--;
            continue;
        }
  }
}
}

void NucleoMotor::LimpiarVector()
{
  // Elimina los elementos del vector
  vector<cSprite*>::iterator ivSprite = m_vcSprites.begin();
  while (ivSprite != m_vcSprites.end())
  {
    delete (*ivSprite);
    ivSprite = m_vcSprites.erase(ivSprite);
    //ivSprite--;
  }
}

cSprite* NucleoMotor::HitTest(int x, int y)
{
  // Bsqueda si un punto
  // coincide con algn sprite

  // Crea un iterador inverso
  vector<cSprite*>::reverse_iterator ivSprite;

  for (ivSprite = m_vcSprites.rbegin(); ivSprite != m_vcSprites.rend(); ivSprite++)
   if ((*ivSprite)->HitTest(x, y))
     return (*ivSprite);

  // No hay coincidencia
  return NULL;
}
bool NucleoMotor::MIDI(MIDI_ACCION mop, LPTSTR szArchivo, bool bReiniciar)
{
    // Si no se oye el sonido MIDI antes de buscar un error de
    // programa recuerde que primero se debe verificar lo siguiente:
    // 1. Que el volumen del sintetizador est en el nivel correcto
    // 2. Que existe el archivo .mid
  MCIERROR mciError;
  switch (mop)
  {
    case PLAYMIDI:
        // Si el dispositivo ya est abierto y se
        // informa el archivo de reproduccin
        // cierra y vuelve a abrir
        if (m_uiMIDIPlayerID != 0 && szArchivo != NULL)
        {
            mciSendCommand(m_uiMIDIPlayerID, MCI_CLOSE, 0, NULL);
            m_uiMIDIPlayerID = 0;
        }
        // Verifica si el dispositivo ya est abierto
        if (m_uiMIDIPlayerID == 0)
        {
            // Estructura utilizada en la opcin Open
            MCI_OPEN_PARMS mciOpenParms;

            // Tipo de dispositivo MIDI
            mciOpenParms.lpstrDeviceType = "sequencer";
            // Nombre del archivo .mid que se reproducir
            mciOpenParms.lpstrElementName = szArchivo;
            mciError = mciSendCommand(NULL, MCI_OPEN, MCI_OPEN_TYPE | MCI_OPEN_ELEMENT,
                (DWORD_PTR)&mciOpenParms);
            // Si la funcin devuelve 0, se ejecut correctamente
            // y tenemos el identificador que utilizaremos
            // en otros comandos posteriores
            // en la estructura nos queda registrado el identificador
            if (mciError == 0)
                m_uiMIDIPlayerID = mciOpenParms.wDeviceID;
            else
              // si no pudo abrir el dispositivo
              // vuelve a la funcin llamante
            return false;
        }
        // Reinicia la reproduccin
        if (bReiniciar)
        {
            MCI_SEEK_PARMS mciSeekParms;
            mciError = mciSendCommand(m_uiMIDIPlayerID, MCI_SEEK, MCI_SEEK_TO_START,
                (DWORD_PTR)&mciSeekParms);
            if (mciError != 0)
            {
                // si no es 0, se cierra el dispositivo
                mciSendCommand(m_uiMIDIPlayerID, MCI_CLOSE, 0, NULL);
                m_uiMIDIPlayerID = 0;
                return false;
            }
        }
        // Reproduccin del archivo MIDI
        MCI_PLAY_PARMS mciPlayParms;
        mciError = mciSendCommand(m_uiMIDIPlayerID, MCI_PLAY, 0,
            (DWORD_PTR)&mciPlayParms);
        if (mciError != 0)
        {
            // si no es 0, se cierra el dispositivo
            mciSendCommand(m_uiMIDIPlayerID, MCI_CLOSE, 0, NULL);
            m_uiMIDIPlayerID = 0;
            return false;
        }
        break;
    case PAUSARMIDI:
        // se intenta pausar la reproduccin
        // (slo si hay iID asignado
        if (m_uiMIDIPlayerID != 0)
            mciSendCommand(m_uiMIDIPlayerID, MCI_PAUSE, 0, NULL);
        break;
    case CERRARMIDI:
        mciSendCommand(m_uiMIDIPlayerID, MCI_CLOSE, 0, NULL);
          m_uiMIDIPlayerID = 0;
        break;
  }
  return true;

}


