//-----------------------------------------------------------------
// Aplicacin de prueba de la clase cSprite
// Archivo de cdigo C++ - TestCSprite.cpp
//-----------------------------------------------------------------

//-----------------------------------------------------------------
// Archivos de inclusin
//-----------------------------------------------------------------
#include "TestCSprite.h"
#include <math.h>

//-----------------------------------------------------------------
// Funciones del NucleoMotor
//-----------------------------------------------------------------
BOOL FJ_InicializarJuego(HINSTANCE hInstancia)
{
  // Crea el ncleo del juego
  g_pJuego = new NucleoMotor(hInstancia,
    TEXT("TestcSprite"), TEXT("TestcSprite"),
    IDI_TESTCSPRITE, IDI_TESTCSPRITE_SM,
    600, 450, 10); // frecuencia 10 ciclos por segundos

  if (g_pJuego == NULL)
    return false;

  // Se inicializa el joystick
  if (!g_pJuego->InicializarJoystick())
    return false;

  return true;
}

void FJ_ComenzarJuego(HWND hWindow)
{
  // Crea una serie de mapa de bits
  HDC hDC = GetDC(hWindow);

  // Crea un mapa de bits a partir de un recurso
  // Los mapas de bits son de profundidad 24 bits
  g_pNumero[0] = new cBMP(hDC, TEXT("Recursos//1.bmp"));
  g_pNumero[1] = new cBMP(hDC, TEXT("Recursos//2.bmp"));
  g_pNumero[2] = new cBMP(hDC, TEXT("Recursos//3.bmp"));
  g_pNumero[3] = new cBMP(hDC, TEXT("Recursos//4.bmp"));
  g_pNumero[4] = new cBMP(hDC, TEXT("Recursos//5.bmp"));
  g_pNumero[5] = new cBMP(hDC, TEXT("Recursos//6.bmp"));

  g_pFondoJuego = new cBMP(hDC, IDB_BMP1, g_pJuego->ObtenerInstancia());

  // Se define un rectngulo para el lmite
  // del movimiento de los sprites: toda la pantalla
  RECT rcAreaAccion = {0, 0, g_pJuego->ObtenerAncho(), g_pJuego->ObtenerAltura()};


  g_pSprite[0] = new cSprite(g_pNumero[0], rcAreaAccion, AFL_REBOTAR);
  g_pSprite[1] = new cSprite(g_pNumero[1], rcAreaAccion, AFL_DETENER);

  // posicin inicial del objeto
  POINT ptInicial =  {400, 200};
  // se define un hot spot interno
  RECT rcHotSpot = {20, 20, 50, 50};
  // velocidad
  POINT ptVelocidad = {5, 5};
  g_pSprite[2] = new cSprite(g_pNumero[2], ptInicial, rcHotSpot, ptVelocidad, 2,
    rcAreaAccion, AFL_REINICIAR);

  g_pSprite[3] = new cSprite(g_pNumero[3], rcAreaAccion, AFL_REAPARECER);
  g_pSprite[4] = new cSprite(g_pNumero[4], rcAreaAccion, AFL_DESAPARECER);

  // limitamos el ltimo rectngulo
  // el nmero 6 rebotar en un rea ms
  // pequea
  rcAreaAccion.right /= 2;
  rcAreaAccion.bottom /= 3;
  g_pSprite[5] = new cSprite(g_pNumero[5], rcAreaAccion, AFL_REBOTAR);


  // Establece el punto de inicio
  g_pSprite[0]->AsignarPosicion(100, 0);
  g_pSprite[1]->AsignarPosicion(200, 0);

  // el nmero 3 ya tiene asignado un punto de inicio especial
  // g_pSprite[4]->AsignarPosicion(400, 0);

  g_pSprite[3]->AsignarPosicion(400, 0);
  g_pSprite[4]->AsignarPosicion(500, 0);
  g_pSprite[5]->AsignarPosicion(0, 100);

  // Establece diferentes velocidades
  g_pSprite[0]->AsignarVelocidad(2, 2);
  g_pSprite[1]->AsignarVelocidad(4, 5);
  g_pSprite[2]->AsignarVelocidad(1, 5);
  g_pSprite[3]->AsignarVelocidad(3, 8);
  g_pSprite[4]->AsignarVelocidad(-1, 1);
  g_pSprite[5]->AsignarVelocidad(2, 0);

  // Establece diferentes orden Z
  // siguiendo el valor de cada nmero
  // es decir, el 2 tapa al 1, el 3 al 1 y
  // al 2, etc.
  g_pSprite[0]->AsignarOrdenZ(0);
  g_pSprite[1]->AsignarOrdenZ(1);
  g_pSprite[2]->AsignarOrdenZ(2);
  g_pSprite[3]->AsignarOrdenZ(3);
  g_pSprite[4]->AsignarOrdenZ(4);
  g_pSprite[5]->AsignarOrdenZ(5);

  // Contamos la cantidad de ciclos
  g_iContBucle = 0;

  g_iQSprites = 6;

  // Determina el mximo orden z
  // que se haya definido
  g_iMaxOrdenZ = 0;
  for (int i = 0; i < g_iQSprites; i++)
    g_iMaxOrdenZ = max(g_pSprite[i]->ObtenerOrdenZ(),g_iMaxOrdenZ);

  // Bfer auxiliar de repintado
  // Crea un contexto de dispositivo y un
  // mapa de bits adicional
  g_hBuferAuxDC = CreateCompatibleDC(hDC);
  g_hBuferAuxBitmap = CreateCompatibleBitmap(hDC,
      g_pJuego->ObtenerAncho(), g_pJuego->ObtenerAltura());
  SelectObject(g_hBuferAuxDC , g_hBuferAuxBitmap);
}

void FJ_FinalizarJuego()
{
  // Elimina los elementos del bfer auxiliar
  DeleteObject(g_hBuferAuxBitmap);
  DeleteObject(g_hBuferAuxDC);

  // Elimina objetos matriz
  for (int i = 0; i < g_iQSprites ;i++)
  {
    delete g_pSprite[i];
    delete g_pNumero[i];
  }
  // elimina el mapa de bits de fondo
  delete g_pFondoJuego;

  // Elimina el objeto
  delete g_pJuego;
}

void FJ_ReactivarJuego(HWND hWindow)
{
    // Capturar el joystick
     g_pJuego->CapturarJoystick();
}

void FJ_DetenerJuego(HWND hWindow)
{
    // Liberar el joystick
     g_pJuego->LiberarJoystick();
}

void FJ_PintarJuego(HDC hDC)
{

    POINT ptOrigen = {0, 0};
    // El fondo del juego se pinta primero
    g_pFondoJuego->Dibujar(hDC, ptOrigen);

    // Despus se pintan los sprites en orden Z
    // de 0 en adelante
    for (int z = 0; z < (g_iMaxOrdenZ+1); z++)
        for (int i = 0; i < g_iQSprites; i++)
           if (g_pSprite[i]->ObtenerOrdenZ() == z)
               g_pSprite[i]->Dibujar(hDC);
}

void FJ_CicloJuego()
{
    // Actualiza las posiciones de los sprites
     for (int i = 0; i < g_iQSprites; i++)
        g_pSprite[i]->ActualizarPosicion();

    ++g_iContBucle;
     FJ_PintarJuego(g_hBuferAuxDC);

    HWND hVentana = g_pJuego->ObtenerVentana();
    HDC hDC = GetDC(hVentana);

    // se utilizan coordenadas 0, 0
    // porque se realiza copia completa
    // de un rea a otra
    BitBlt(hDC,
        0, 0, g_pJuego->ObtenerAncho(), g_pJuego->ObtenerAltura(),
        g_hBuferAuxDC, 0, 0,
        SRCCOPY);

    ReleaseDC(hVentana, hDC);
}
void FJ_LButtonDown(LPARAM lParam)
{
    POINT ptCoord = {LOWORD(lParam), HIWORD(lParam)};

  // Actualiza las posiciones de los sprites
     for (int i = 0; i < g_iQSprites; i++)
     {
       HITLEVEL hlHit = g_pSprite[i]->HitTest(ptCoord.x, ptCoord.y);
       switch (hlHit)
       {
           case HT_NOHOTSPOT:
                g_pSprite[i]->AsignarPosicion(0,0);
                break;
           case HT_HOTSPOT:
                g_pSprite[i]->AsignarPosicion(600,0);
                break;
        }
     }
}
void FJ_RButtonUp(LPARAM lParam)
{
}
void FJ_LButtonUp(LPARAM lParam)
{
}
void FJ_MouseMove(LPARAM lParam)
{
}

void FJ_RButtonDown(LPARAM lParam)
{
}

void FJ_CheckTeclado()
{
}
void FJ_CheckJoystick(WORD wJoyEstado)
{
}

