//-----------------------------------------------------------------
// Archivo de cdigo C++ - ColisionesControladas.cpp
//-----------------------------------------------------------------
//-----------------------------------------------------------------
// Archivos de inclusin
//-----------------------------------------------------------------
#include "ColisionesControladas.h"
#include <math.h>

//-----------------------------------------------------------------
// Funciones del NucleoMotor
//-----------------------------------------------------------------
BOOL FJ_InicializarJuego(HINSTANCE hInstancia)
{
  // Crea el ncleo del juego
  g_pJuego = new NucleoMotor(hInstancia,
    TEXT("ColisionesControladas"), TEXT("ColisionesControladas"),
    IDI_COLISIONES, IDI_COLISIONES_SM,
    640, 480, 12); // frecuencia  fps

  if (g_pJuego == NULL)
    return false;

  // Se inicializa el joystick
  if (!g_pJuego->InicializarJoystick())
    return false;

  return true;
}
void FJ_ComenzarJuego(HWND hWindow)
{
  // Crea una serie de mapa de bits
  HDC hDC = GetDC(hWindow);

  // el objeto impacto se mostrar cada vez que
  // se produzca una colisin
  g_pImpacto = new cBMP(hDC, TEXT("Recursos//Impacto.bmp"));

  g_pFondoJuego = new cBMP(hDC, IDB_BMP1, g_pJuego->ObtenerInstancia());

  // Crea un mapa de bits a partir de archivos
  // Los mapas de bits son de profundidad 24 bits
  g_pPelota[0] = new cBMP(hDC, TEXT("Recursos//1.bmp"));
  g_pPelota[1] = new cBMP(hDC, TEXT("Recursos//2.bmp"));
  g_pPelota[2] = new cBMP(hDC, TEXT("Recursos//3.bmp"));
  g_pPelota[3] = new cBMP(hDC, TEXT("Recursos//4.bmp"));
  g_pPelota[4] = new cBMP(hDC, TEXT("Recursos//5.bmp"));
  g_pPelota[5] = new cBMP(hDC, TEXT("Recursos//6.bmp"));
  g_pPelota[6] = new cBMP(hDC, TEXT("Recursos//7.bmp"));
  g_pPelota[7] = new cBMP(hDC, TEXT("Recursos//8.bmp"));

  // Se define un rectngulo para el lmite
  // del movimiento de los sprites: toda la pantalla
  RECT rcAreaAccion = {0, 0, g_pJuego->ObtenerAncho(), g_pJuego->ObtenerAltura()};

  // Definimos un hot spot menor que el
  // rectgulo del mapa de bits
  // los mapas de bits son de 50x50
  // por lo tanto, el rea sensible
  // es algo menor
  RECT rcHotSpot = {5, 5, 45, 45};

  cSprite* pcsTemp;
  // creacin de los sprites
  // y carga de los sprites en un vector
  // que nos facilita la gestin de los
  // sprites
  for (int i = 0; i< 7; i++)
  {
    // al objeto 0 se le asigna la accin de destruirse al tocar algn
    // lmite, al resto a rebotar
    // objeto 0 es el crculo azul
    if ( i == 0)
       pcsTemp = new cSprite(g_pPelota[i], rcAreaAccion, AFL_DESTRUIR);
    else
       pcsTemp = new cSprite(g_pPelota[i], rcAreaAccion, AFL_REBOTAR);

    // Asignamos la velocidad del sprite
    pcsTemp->AsignarVelocidad(10,15);
    // Se informa el hot spot del sprite
    pcsTemp->AsignarHotSpot(rcHotSpot);

    // Asignamos un mapa de bits asociado
    // para mostrar en caso de impacto
    pcsTemp->AsignarcBmpImpacto(g_pImpacto);

    // Se asigna un identificador a cada objeto
    pcsTemp->AsignarID(i);

    // Se guarda el puntero en un vector
    g_pJuego->AgregarSprite(pcsTemp);
  }
  // este objeto se debe destruir al tocar un lmite
  // se le asigna un identificar para diferenciarlo
  // ya que quien impacte con este objeto quedar
  // destruido
  pcsTemp = new cSprite(g_pPelota[7], rcAreaAccion, AFL_REBOTAR);
  pcsTemp->AsignarID(7);
  pcsTemp->AsignarVelocidad(1,1);
  pcsTemp->AsignarPosicion(60,60);
  pcsTemp->AsignarHotSpot(rcHotSpot);
  pcsTemp->AsignarcBmpImpacto(g_pImpacto);

  // Se guarda el puntero en un vector
  g_pJuego->AgregarSprite(pcsTemp);

  // Contamos la cantidad de ciclos
  g_iContBucle = 0;

  // Bfer auxiliar de repintado
  // Crea un contexto de dispositivo y un
  // mapa de bits adicional
  g_hBuferAuxDC = CreateCompatibleDC(hDC);
  g_hBuferAuxBitmap = CreateCompatibleBitmap(hDC,
      g_pJuego->ObtenerAncho(), g_pJuego->ObtenerAltura());
  SelectObject(g_hBuferAuxDC , g_hBuferAuxBitmap);
}

void FJ_FinalizarJuego()
{
  // Elimina los elementos del bfer auxiliar
  DeleteObject(g_hBuferAuxBitmap);
  DeleteObject(g_hBuferAuxDC);

  // Elimina objetos bmp (pelotas)
  for (int i = 0; i < 8 ;i++)
    delete g_pPelota[i];

  // Bitmap de impacto
  delete g_pImpacto;

   // elimina el mapa de bits de fondo
  delete g_pFondoJuego;

  // Limpieza del vector de sprites
  g_pJuego->LimpiarVector();

 

  // Elimina el objeto
   delete g_pJuego;
}
void FJ_ReactivarJuego(HWND hWindow)
{
    // Capturar el joystick
     g_pJuego->CapturarJoystick();
}
void FJ_DetenerJuego(HWND hWindow)
{
    // Liberar el joystick
     g_pJuego->LiberarJoystick();
}
void FJ_PintarJuego(HDC hDC)
{

    POINT ptOrigen = {0, 0};
    // El fondo del juego se pinta primero
    g_pFondoJuego->Dibujar(hDC, ptOrigen);

    // Ahora dibuja los sprites desde el vector
    // que pertenece al NucleoMotor
    g_pJuego->DibujarSprites(hDC);
}
void FJ_CicloJuego()
{
    // Actualiza las posiciones de los sprites
    g_pJuego->ActualizarSprites();

    ++g_iContBucle;
    // Pintar en el contexto de dispositivo
    // auxiliar para eliminar
    // la sensacin de parpadeo
     FJ_PintarJuego(g_hBuferAuxDC);

    // Pintura real de la pantalla en una nica operacin
    HWND hVentana = g_pJuego->ObtenerVentana();
    HDC hDC = GetDC(hVentana);

    // se utilizan coordenadas 0, 0
    // porque se realiza copia completa
    // de un rea a otra
    BitBlt(hDC,
        0, 0, g_pJuego->ObtenerAncho(), g_pJuego->ObtenerAltura(),
        g_hBuferAuxDC, 0, 0,
        SRCCOPY);

    ReleaseDC(hVentana, hDC);
}
void FJ_LButtonDown(LPARAM lParam)
{
  POINT ptCoord = {LOWORD(lParam), HIWORD(lParam)};

  // Verifica si algn sprite est en esa posicin
  cSprite* pcsHit = g_pJuego->HitTest(ptCoord.x, ptCoord.y);

  // pcsHit no es nulo es porque hay un sprite en ese punto
  // donde actu el ratn
  if (pcsHit)
  {
      // comprobamos el nivel del hit
      HITLEVEL hlHit = pcsHit->HitTest(ptCoord.x, ptCoord.y);
       switch (hlHit)
       {
           case HT_NOHOTSPOT:
           // si no es hot spot se manda al objeto a la esquina superior izquierda
                pcsHit->AsignarPosicion(0,0);
                break;
           case HT_HOTSPOT:
            // si es hot spot se manda al objeto a la esquina inferior derecha
                pcsHit->AsignarPosicion(600,400);
                break;
        }
  }
}
void FJ_RButtonUp(LPARAM lParam)
{
}
void FJ_LButtonUp(LPARAM lParam)
{
}
void FJ_MouseMove(LPARAM lParam)
{
}
void FJ_RButtonDown(LPARAM lParam)
{
}
void FJ_CheckTeclado()
{
}
void FJ_CheckJoystick(WORD wJoyEstado)
{
}
APC_ACCION  FJ_NotificarColision(TC_COLISION tcColision, cSprite* pSprite, cSprite* pSpriteImpactado)
{
  POINT ptVi;
  switch (tcColision)
  {
        case TC_SPRITE:
            // Colisiones entre sprites
            // se invierten las velocidades
            ptVi = pSpriteImpactado->ObtenerVelocidad();
            pSpriteImpactado->AsignarVelocidad(pSprite->ObtenerVelocidad());
            pSprite->AsignarVelocidad(ptVi);
            if ((pSpriteImpactado->ObtenerID() == 7) )
                return APC_DESTRUIR;
            break;
        case TC_LIMITES:
            // Colisin de un sprite contra sus lmites
            break;
        default:
            break;
  }

  return APC_NOACCION;
}

