//-----------------------------------------------------------------
// Clase cSprite
// Cdigo fuente - cSprite.cpp
//-----------------------------------------------------------------

//-----------------------------------------------------------------
// Archivos de inclusin
//-----------------------------------------------------------------
#include "CSprite.h"

//-----------------------------------------------------------------
// cSprite Constructores y Destructor
//-----------------------------------------------------------------
cSprite::cSprite(cBMP* pCBmp)
{
  // Asignacin de valores
  m_pcBmp = pCBmp;

  // Valores predeterminados

  // define el rectngulo externo del sprite
  // por lo tanto la posicin predetrminada ser 0, 0
  SetRect(&m_rcSprite, 0, 0,
        pCBmp->ObtenerAncho(),
        pCBmp->ObtenerAltura());

  // Punto inicial del sprite
  m_ptInicial.x = m_ptInicial.y = 0;

  // define el rectngulo sensible del sprite
  // de manera predeterminada es todo el mapa de bits
  CopyRect(&m_rcHotSpotDef, &m_rcSprite);

  // sprite quieto
  m_ptVelocidad.x = m_ptVelocidad.y = 0;

  // Orden de dibujo
  m_iOrdenZ = 0;

  // mbito de accin predeterminada
  // totalmente limitada al propio sprite
  CopyRect(&m_rcAreaAccion, &m_rcSprite);

  // De modo predeterminado, cuando el objeto
  // toque un lmite, se detendr
  m_wAccionFinArea = AFL_DETENER;

  // Indicador de impacto
  m_bImpacto = false;

  // Bitmap de impacto
  m_pcBmpImpacto = NULL;


}

cSprite::cSprite(cBMP* pCBmp, RECT& rcAreaAccion, AFL_ACCION wAccionFinArea)
{

  // Posicionamiento aleatorio

  int iX = rand() % (rcAreaAccion.right - rcAreaAccion.left);
  int iY = rand() % (rcAreaAccion.bottom - rcAreaAccion.top);

  // Inicializa las variables miembros
  m_pcBmp = pCBmp;

  // define el rectngulo externo
  // en un punto aleatorio
  SetRect(&m_rcSprite, iX, iY,
    iX + pCBmp->ObtenerAncho(),
    iY + pCBmp->ObtenerAltura());

  // Punto inicial del sprite
  m_ptInicial.x = iX;
  m_ptInicial.y = iY;

  // define el rectngulo sensible del sprite
  // de manera predeterminada es todo el mapa de bits
  // el rectngulo sensible es relativo al rectngulo
  // exterior
  SetRect(&m_rcHotSpotDef, 0, 0,
    m_rcSprite.right - m_rcSprite.left,
    m_rcSprite.bottom - m_rcSprite.top);

  // velocidad = 0
  m_ptVelocidad.x = m_ptVelocidad.y = 0;

  // orden 0
  m_iOrdenZ = 0;

  // mbito de accin
  CopyRect(&m_rcAreaAccion, &rcAreaAccion);

  // Accin al llegar al lmite
  m_wAccionFinArea = wAccionFinArea;

  // Indicador de impacto
  m_bImpacto = false;

  // Bitmap de impacto
  m_pcBmpImpacto = NULL;
}


cSprite::cSprite(cBMP* pCBmp,
    RECT& rcSprite, RECT& rcHotSpot,
    POINT ptVelocidad, int iOrdenZ,
    RECT& rcAreaAccion, AFL_ACCION wAccionFinArea)
{
  // Constructor ms completo
  // Inicializa las variables miembro
  m_pcBmp = pCBmp;

  // define el rectngulo externo a partir de un rectngulo
  CopyRect(&m_rcSprite, &rcSprite);

   // Punto inicial del sprite
  m_ptInicial.x = m_rcSprite.left;
  m_ptInicial.y = m_rcSprite.top;

  // Velocidad
  m_ptVelocidad = ptVelocidad;

  // Orden Z
  m_iOrdenZ = iOrdenZ;

  // rea de movimiento
  CopyRect(&m_rcAreaAccion, &rcAreaAccion);

  // Accin al llegar al fin del rea de movimiento
  m_wAccionFinArea = wAccionFinArea;

  // rea sensible del mapa de bits
  // el rectngulo tiene coordenadas
  // relativas al rectngulo externo
  CopyRect(&m_rcHotSpotDef, &rcHotSpot);

  // Indicador de impacto
  m_bImpacto = false;

  // Bitmap de impacto
  m_pcBmpImpacto = NULL;
}

cSprite::cSprite(cBMP* pCBmp,
    POINT ptPosition, RECT& rcHotSpot,
    POINT ptVelocidad, int iOrdenZ,
    RECT& rcAreaAccion, AFL_ACCION wAccionFinArea)
{
  // Constructor ms completo
  // Inicializa las variables miembro
  m_pcBmp = pCBmp;

  // define el rectngulo externo a partir de un punto
  // y de las medidas del mapa de bits
  SetRect(&m_rcSprite, ptPosition.x, ptPosition.y,
    ptPosition.x + pCBmp->ObtenerAncho(), ptPosition.y + pCBmp->ObtenerAltura());

  // Punto inicial del sprite
  m_ptInicial.x = m_rcSprite.left;
  m_ptInicial.y = m_rcSprite.top;

  // Velocidad
  m_ptVelocidad = ptVelocidad;

  // Orden Z
  m_iOrdenZ = iOrdenZ;

  // rea de movimiento
  CopyRect(&m_rcAreaAccion, &rcAreaAccion);

  // Accin al llegar al fin del rea de movimiento
  m_wAccionFinArea = wAccionFinArea;

  // rea sensible del mapa de bits
  // el rectngulo tiene coordenadas
  // relativas al rectngulo externo
  CopyRect(&m_rcHotSpotDef, &rcHotSpot);

  // Indicador de impacto
  m_bImpacto = false;

  // Bitmap de impacto
  m_pcBmpImpacto = NULL;
}

cSprite::~cSprite()
{
}

//-----------------------------------------------------------------
// Mtodos generales
//-----------------------------------------------------------------
APC_ACCION cSprite::ActualizarPosicion()
{
  // Actualiza posicin del sprite
  POINT ptPosicionActual;
  SIZE  szSprite;

  // La posicin del objeto sprite
  ptPosicionActual.x = m_rcSprite.left + m_ptVelocidad.x;
  ptPosicionActual.y = m_rcSprite.top + m_ptVelocidad.y;

  POINT ptAnterior = ptPosicionActual;

  // ancho y alto del sprite
  szSprite.cx =  m_rcSprite.right - m_rcSprite.left;
  szSprite.cy =  m_rcSprite.bottom - m_rcSprite.top;

  // asignar una accin predeterminada
  APC_ACCION apcRet = APC_NOACCION;

  POINT ptInversionVelocidad;
  // Comprueba si toca lmites
  switch (m_wAccionFinArea)
  {
    case AFL_DESAPARECER:
        // corrige eje x
        // para que el sprite quede a un costado
        if ((ptPosicionActual.x + szSprite.cx) < m_rcAreaAccion.left)
            ptPosicionActual.x = m_rcAreaAccion.left - szSprite.cx;
        else if (ptPosicionActual.x  > m_rcAreaAccion.right)
            ptPosicionActual.x = m_rcAreaAccion.right + szSprite.cx;
        // corrige eje y
        // para que el sprite quede a un costado
        if (ptPosicionActual.y + szSprite.cy < m_rcAreaAccion.top)
            ptPosicionActual.y = m_rcAreaAccion.bottom + szSprite.cy;
        else if (ptPosicionActual.y > m_rcAreaAccion.bottom)
            ptPosicionActual.y = m_rcAreaAccion.top - szSprite.cy;
        break;
    case AFL_REAPARECER:
        // corrige eje x
        // para que el sprite reaparezca por el otro lado
        if ((ptPosicionActual.x) < m_rcAreaAccion.left)
            ptPosicionActual.x = m_rcAreaAccion.right;
        else if (ptPosicionActual.x  > m_rcAreaAccion.right)
            ptPosicionActual.x = m_rcAreaAccion.left;
        // corrige eje y
        // para que el sprite reaparezca por el otro lado
        if (ptPosicionActual.y < m_rcAreaAccion.top)
            ptPosicionActual.y = m_rcAreaAccion.bottom;
        else if (ptPosicionActual.y > m_rcAreaAccion.bottom)
            ptPosicionActual.y = m_rcAreaAccion.top;
        break;
    case AFL_REINICIAR:
        // corrige eje x
        // para que el sprite reaparezca el punto orginal
        if ((ptPosicionActual.x < m_rcAreaAccion.left) ||
            ((ptPosicionActual.x + szSprite.cx)  >  m_rcAreaAccion.right) ||
            (ptPosicionActual.y < m_rcAreaAccion.top) ||
            ((ptPosicionActual.y + szSprite.cy) > m_rcAreaAccion.bottom))
             ptPosicionActual = this->ObtenerPosicionInicial();
        break;
    case AFL_REBOTAR:

        ptInversionVelocidad = m_ptVelocidad;
        if (ptPosicionActual.x < m_rcAreaAccion.left)
        {
            // corrige la posicin
            // cambia de direccin
            ptPosicionActual.x = m_rcAreaAccion.left;
            ptInversionVelocidad.x = -ptInversionVelocidad.x;
        }
        else if ((ptPosicionActual.x + szSprite.cx ) > m_rcAreaAccion.right)
        {
            ptPosicionActual.x = m_rcAreaAccion.right - szSprite.cx;
            ptInversionVelocidad.x = -ptInversionVelocidad.x;
        }
        if (ptPosicionActual.y < m_rcAreaAccion.top)
        {
          ptPosicionActual.y = m_rcAreaAccion.top;
          ptInversionVelocidad.y = -ptInversionVelocidad.y;
        }
        else if ((ptPosicionActual.y + szSprite.cy) > m_rcAreaAccion.bottom)
        {
          ptPosicionActual.y = m_rcAreaAccion.bottom - szSprite.cy;
          ptInversionVelocidad.y = -ptInversionVelocidad.y;
        }
        AsignarVelocidad(ptInversionVelocidad);
        break;
    case AFL_DETENER:
        if (ptPosicionActual.x  < m_rcAreaAccion.left)
        {
            ptPosicionActual.x = m_rcAreaAccion.left;
            AsignarVelocidad(0,0);
        }
        if ((ptPosicionActual.x + szSprite.cx)  > m_rcAreaAccion.right)
        {
            ptPosicionActual.x = m_rcAreaAccion.right - szSprite.cx;
            AsignarVelocidad(0,0);
        }
        if (ptPosicionActual.y  < m_rcAreaAccion.top)
        {
            ptPosicionActual.y = m_rcAreaAccion.top;
            AsignarVelocidad(0,0);
        }
        if ((ptPosicionActual.y + szSprite.cy)  > m_rcAreaAccion.bottom)
        {
            ptPosicionActual.y = m_rcAreaAccion.bottom - szSprite.cy;
            AsignarVelocidad(0,0);
        }
        break;
    case AFL_DESTRUIR:
        if ((ptPosicionActual.x  < m_rcAreaAccion.left) ||
           ((ptPosicionActual.x + szSprite.cx)  > m_rcAreaAccion.right) ||
           (ptPosicionActual.y  < m_rcAreaAccion.top) ||
           ((ptPosicionActual.y + szSprite.cy)  > m_rcAreaAccion.bottom))
            return APC_DESTRUIR;
        break;
    }
    AsignarPosicion(ptPosicionActual);
    // devuelve la accin que se debe realizar sobre
    // el objeto
    // si hubo correcin de posicin
    // es porque se lleg a un lmite
    if (ptPosicionActual.x != ptAnterior.x ||
        ptPosicionActual.y != ptAnterior.y)
        return APC_NOTIFICAR;
    return apcRet;;
}

void cSprite::Dibujar(HDC hDC)
{
  // Dibuja el sprite
  if (m_pcBmp != NULL)
  {
    POINT ptPos = { m_rcSprite.left, m_rcSprite.top};
    m_pcBmp->Dibujar(hDC,ptPos, TRUE);
    if (m_bImpacto && (m_pcBmpImpacto !=  NULL))
    {
        POINT ptPos = {m_rcImpacto.left, m_rcImpacto.top};
        m_pcBmpImpacto->Dibujar(hDC, ptPos, true);
    }
    m_bImpacto = false;
  }
}
//-----------------------------------------------------------------
// Mtodos
//-----------------------------------------------------------------
HITLEVEL cSprite::HitTest(int x, int y)
{
  POINT ptPunto;
  ptPunto.x = x;
  ptPunto.y = y;
  RECT rcHotSpotTemp;
  SetRect(&rcHotSpotTemp,
    m_rcSprite.left + m_rcHotSpotDef.left,
    m_rcSprite.top + m_rcHotSpotDef.top,
    m_rcSprite.left + m_rcHotSpotDef.right,
    m_rcSprite.top + m_rcHotSpotDef.bottom);

  if (PtInRect(&rcHotSpotTemp, ptPunto))
    return HT_HOTSPOT;
  else if  (PtInRect(&m_rcSprite, ptPunto))
        return HT_NOHOTSPOT;
  else  return HT_NO;
}

//-----------------------------------------------------------------
//-----------------------------------------------------------------
void cSprite::AsignarPosicion(POINT ptPosicion)
{
  // Utiliza la estructura POINT para
  // realizar un desplazamiento del rectngulo del sprite
  OffsetRect(&m_rcSprite, ptPosicion.x - m_rcSprite.left,
    ptPosicion.y - m_rcSprite.top);
}
void cSprite::AsignarPosicion(int x, int y)
{
  // Desplaza el rectngulo con coordenadas x y
  OffsetRect(&m_rcSprite, x - m_rcSprite.left, y - m_rcSprite.top);
}


void cSprite::AsignarVelocidad(int x, int y)
{
  // Asigna nuevos valores a la estructura POINT
  // con dos valores
  m_ptVelocidad.x = x;
  m_ptVelocidad.y = y;

}

void cSprite::AsignarVelocidad(POINT ptVelocidad)
{
  // Asigna nuevos valores a la estructura POINT
  // con un valor de tipo POINT
  m_ptVelocidad.x = ptVelocidad.x;
  m_ptVelocidad.y = ptVelocidad.y;

}
// Obtiene rectngulo hot spot
RECT&   cSprite::ObtenerHotSpot()
{
   // se recalcula la posicin del
   // rectngulo hot spot
   SetRect(&m_rcHotSpot,
     m_rcSprite.left + m_rcHotSpotDef.left,
     m_rcSprite.top + m_rcHotSpotDef.top,
     m_rcSprite.left + m_rcHotSpotDef.right,
     m_rcSprite.top + m_rcHotSpotDef.bottom);
   return  m_rcHotSpot;
}


 BOOL cSprite::VerificarColision(cSprite* pObjcSprite)
{
  // compara el objeto cSprite vigente
  // con otro recibido por parmetro del
  // mtodo
  RECT& rcTest = pObjcSprite->ObtenerHotSpot();
  RECT& rcThis =this->ObtenerHotSpot();
  // True indica que al menos un vrtice de un rectngulo
  // est tocando o adentro del otro: colisin
  m_bImpacto = IntersectRect(&m_rcImpacto, &rcThis, &rcTest);
  return m_bImpacto;
}



