//-----------------------------------------------------------------
// Objeto cSprite
// Archivo de encabezado - cSprite.h
//-----------------------------------------------------------------

#pragma once

//-----------------------------------------------------------------
// Archivos de inclusin
//-----------------------------------------------------------------
#include <windows.h>
#include "cBMP.h"
#include <math.h>

//-----------------------------------------------------------------
// Constantes para control de hit
//-----------------------------------------------------------------
typedef WORD        HITLEVEL;
const   HITLEVEL    HT_NO           = 0,
                    HT_NOHOTSPOT    = 1,
                    HT_HOTSPOT      = 2;
//-----------------------------------------------------------------
// Constantes de opciones al llegar al final del limite (AFL_)
//-----------------------------------------------------------------
typedef WORD        AFL_ACCION;
const   AFL_ACCION  AFL_DETENER     = 0,
                    AFL_REBOTAR     = 1,
                    AFL_REAPARECER  = 2,
                    AFL_DESAPARECER = 3,
                    AFL_REINICIAR   = 4,
                    AFL_DESTRUIR    = 5;

//-----------------------------------------------------------------
// Constantes de acciones al producirse una colisin (APC_)
//-----------------------------------------------------------------
typedef WORD        APC_ACCION;
const   APC_ACCION  APC_NOACCION  = 0,
                    APC_NOTIFICAR = 1, // Notificar
                    APC_DESTRUIR  = 2, // Destruye 1 objeto
                    APC_DESTRUIR2 = 3; // Destruye los dos objetos
//-----------------------------------------------------------------
// Constantes de tipos de colisiones (TC_)
//-----------------------------------------------------------------
typedef WORD        TC_COLISION;
const   TC_COLISION TC_NOCOLISION  = 0,
                    TC_LIMITES     = 1, // Colisin contra lmite
                    TC_SPRITE      = 2; // Colisin contra sprite
//-----------------------------------------------------------------
// Clase cSprite
//-----------------------------------------------------------------
class cSprite
{
protected:
  // Variables miembro protegidos

  cBMP*         m_pcBmp;

  // Rectngulo externo del mapa de bits
  RECT          m_rcSprite;

  // Posicin inicial del sprite
  POINT         m_ptInicial;

  // Velocidad en eje x y velocidad en eje y
  // expresado con un objeto POINT
  POINT         m_ptVelocidad;

  // Valor entero representa orden z
  // valor 0 es el fondo y un valor mayor se superpone
  int           m_iOrdenZ;

  // rea rectangular por donde se permite
  // el movimiento del sprite
  RECT          m_rcAreaAccion;

  // Definicin de accin automtica del sprite al
  // tocar lmites
  AFL_ACCION    m_wAccionFinArea;

   // Rectngulo interior del mapa de bits
  // se utiliza para deteccin de colisin
  // Los puntos son relativos al
  // rectngulo externo
  RECT          m_rcHotSpotDef;
  // rectngulo en posiciones respecto a
  // coordenadas de pantalla
  RECT          m_rcHotSpot;

  // rea de impacto
  RECT          m_rcImpacto;

  // Indicador de impacto
  BOOL          m_bImpacto;

  // Bitmap de impacto
  cBMP*         m_pcBmpImpacto;

  // Identificador del sprite
  int           m_iID;



public:
  // Constructores y Destructor

  // Este constructor asume
  // area de accin = pantalla del juego
  // posicin inicial = 0, 0
  // accin al llegar al final del rea = se detiene
  cSprite(cBMP* pCBmp);

  cSprite(cBMP* pCBmp,
    RECT& rcAreaAccion,
    AFL_ACCION wAccionFinArea = AFL_DETENER);

  // Constructor para indicar una posicin inicial
  // informada como un rectngulo y los
  // principales parmetros del objeto cSprite
  cSprite(cBMP* pCBmp,
    RECT& rcSprite,
    RECT& rcHotSpot,
    POINT  ptVelocidad,
    int iOrdenZ,
    RECT& rcAreaAccion,
    AFL_ACCION wAccionFinArea = AFL_DETENER);

  cSprite(cBMP* pCBmp,
    POINT ptPosition,
    RECT& rcHotSpot,
    POINT ptVelocidad,
    int iOrdenZ,
    RECT& rcAreaAccion,
    AFL_ACCION wAccionFinArea = AFL_DETENER);

  virtual ~cSprite();



  // Mtodos
  virtual APC_ACCION  ActualizarPosicion();
  void          Dibujar(HDC hDC);
  HITLEVEL      HitTest(int x, int y);
  BOOL          VerificarColision(cSprite* pObjcSprite1);

  // Mtodos de acceso a las propiedades
  POINT   ObtenerPosicionInicial()      { return m_ptInicial; };

  // Asignacin y obtencin de la posicin del sprite
  // Tres modos de asignar posicin: RECT, POINT y coordenadas x y
  void    AsignarPosicion(POINT ptPosicion);
  void    AsignarPosicion(int x, int y);
  void    AsignarPosicion(RECT& rcPosition)
    { CopyRect(&m_rcSprite, &rcPosition); };
  RECT&   ObtenerPosicion()             { return m_rcSprite; };

  // Asignacin del rectngulo hot spot
  void    AsignarHotSpot(RECT& rcHotSpot)
    { CopyRect(&m_rcHotSpotDef, &rcHotSpot); };
  RECT&   ObtenerHotSpot();


  // Asignacin y obtencin de la velocidad
  // Dos modos de asignar velocidad: x-y  o POINT
  void    AsignarVelocidad(int x, int y);
  void    AsignarVelocidad(POINT ptVelocidad);
  POINT   ObtenerVelocidad()             { return m_ptVelocidad; };

  // Asignacin y obtencin del Orden Z
  void    AsignarOrdenZ(int iOrdenZ)    { m_iOrdenZ = iOrdenZ; };
  int     ObtenerOrdenZ()               { return m_iOrdenZ; };

  // Control de lmites
  void    AsignarAreaAccion(RECT& rcAreaAccion) { CopyRect(&m_rcAreaAccion, &rcAreaAccion); };
  void    AsignarAccionFinArea(AFL_ACCION afl) { m_wAccionFinArea = afl; };

  // Tamao del sprite
  // son los valores del mapa de bits cBmp
  int     ObtenerAncho()                { return m_pcBmp->ObtenerAncho(); };
  int     ObtenerAltura()               { return m_pcBmp->ObtenerAltura(); };


  // Asignacin y obtencin del rectngulo de impacto
  void    AsignarImpacto(RECT& rcPosition)
    { CopyRect(&m_rcImpacto, &rcPosition); };
  RECT&   ObtenerImpacto()              { return m_rcImpacto; };

  // Asignacin y obtencin del indicador de impacto
  void    AsignarIndicadorImpacto(BOOL bIndicador) {m_bImpacto = bIndicador;};
  BOOL    ObtenerIndicadorImpacto()     { return  m_bImpacto;};

  // Asignar Bitmap de impacto
  void    AsignarcBmpImpacto(cBMP* pcBmp) {m_pcBmpImpacto = pcBmp;};
  cBMP*   ObtenercBmpImpacto()          { return  m_pcBmpImpacto;};

   // Asignar Bitmap de impacto
  void  AsignarID(int iID) {m_iID = iID;};
  int   ObtenerID()        { return  m_iID;};
};

